// By. Joey Clemente

#version 330                          
out vec4 FragColor;

in vec3 Color;
in vec3 WorldPos;
in vec3 WorldNormal;

uniform vec3 uLightPos;
uniform vec3 uLightColor;
uniform vec3 uEyePos;
uniform float uAmbientStrength;
uniform float uSpecularStrength;
uniform float uDiffuseStrength;
uniform int uShininess;
uniform vec3 uObjectColor;

uniform vec3 colZero;
uniform vec3 colOne;
uniform vec3 colTwo;
uniform vec3 colThree;
uniform vec3 colFour;
uniform vec3 colFive;

void main(){          
    // Ambient
    vec3 Ambient = uAmbientStrength * uLightColor;

    // Diffuse
    vec3 toLight = normalize(uLightPos - WorldPos);
    vec3 normal = normalize(cross(dFdx(WorldPos), dFdy(WorldPos)));
    float d = max(dot(normal, toLight), 0);

    vec3 Diffuse = uLightColor * d * uDiffuseStrength;

    // Specular
    vec3 toEye = normalize(uEyePos - WorldPos);
    vec3 reflectDirection = reflect(-toLight, WorldNormal);
    float s = pow(max(dot(reflectDirection, toEye), 0), uShininess);

    vec3 Specular = uLightColor * s * uSpecularStrength;

    vec3 newColor;

    if (WorldPos.y > 3.0f)
    {
        newColor = colZero;
    }
    else if (WorldPos.y > 2.0f)
    {
        newColor = colOne;
    }
    else if (WorldPos.y > 1.6f)
    {
        newColor = colTwo;
    }
    else if (WorldPos.y > 1.3f)
    {
        newColor = colThree;
    }
    else if (WorldPos.y > 1.1f)
    {
        newColor = colFour;
    }
    else
    {
        newColor = colFive;
    }

    FragColor = vec4(newColor * (Ambient + Diffuse + Specular), 1.0f);
}
