// By. Joey Clemente

#version 330                          
layout (location = 0) in vec3 in_Pos;  
layout (location = 1) in vec3 in_Color;
layout (location = 2) in vec2 in_TexCoords; 
layout (location = 3) in vec3 in_Normals;

out vec3 Color;
out vec3 WorldPos;
out vec3 WorldNormal;

vec3 newColor = vec3(0,0,0);

uniform mat4 Model;
uniform mat4 View;
uniform mat4 Projection;
uniform sampler2D noise;
uniform float frequency;
uniform float amplitude;
uniform int octaves;
uniform float persistence;
uniform float lacunarity;

vec3 generatePosition(float x, float y, float z, float amplitude, float frequency, int octaves, float persistence, float lacunarity)
{

    vec3 newPos;

    float noiseHeight = 0;
    float maxLocalHeight = 3.5f;
    float minLocalHeight = -1.75f;

    float sampleX = x * frequency;
    float sampleZ = z * frequency;

    for (int i = 0; i < octaves; i++)
    {
        float perlinValue = texture(noise, vec2(sampleX, sampleZ)).r;
        noiseHeight += perlinValue * amplitude * persistence;
        amplitude *= lacunarity;
        frequency *= lacunarity;

		if (noiseHeight > maxLocalHeight) {
			noiseHeight = maxLocalHeight;
		} 
		if (noiseHeight < minLocalHeight) {
			noiseHeight = minLocalHeight;
		}
    }

    newPos = vec3(sampleX, noiseHeight, sampleZ);

    return newPos;
}

void main(){   
    Color = in_Color;
    gl_Position = Projection * View * Model * vec4(generatePosition(in_Pos.x, in_Pos.y, in_Pos.z, amplitude, frequency, octaves, persistence, lacunarity),1.0);
    WorldPos = vec3(Model * vec4(generatePosition(in_Pos.x, in_Pos.y, in_Pos.z, amplitude, frequency, octaves, persistence, lacunarity),1.0));
    WorldNormal = mat3(transpose(inverse(Model))) * in_Normals;
}
