// By. Joey Clemente

#version 330                          
out vec4 FragColor;

in vec3 Color;
in vec3 WorldPos;
in vec3 WorldNormal;

uniform vec3 uLightPos;
uniform vec3 uLightColor;
uniform vec3 uEyePos;
uniform float uAmbientStrength;
uniform float uSpecularStrength;
uniform float uDiffuseStrength;
uniform int uShininess;
uniform vec3 uObjectColor;

uniform vec3 colZero;
uniform vec3 colOne;
uniform vec3 colTwo;
uniform vec3 colThree;
uniform vec3 colFour;
uniform vec3 colFive;

uniform vec3 planeColor;

void main(){          
    // Ambient
    vec3 Ambient = uAmbientStrength * uLightColor;

    // Diffuse
    vec3 toLight = normalize(uLightPos - WorldPos);
    vec3 normal = normalize(cross(dFdx(WorldPos), dFdy(WorldPos)));
    float d = max(dot(normal, toLight), 0);

    vec3 Diffuse = uLightColor * d * uDiffuseStrength;

    // Specular
    vec3 toEye = normalize(uEyePos - WorldPos);
    vec3 reflectDirection = reflect(-toLight, WorldNormal);
    float s = pow(max(dot(reflectDirection, toEye), 0), uShininess);

    vec3 Specular = uLightColor * s * uSpecularStrength;

    FragColor = vec4(planeColor * (Ambient + Diffuse + Specular), 0.65f);
}
